"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const docGen = tslib_1.__importStar(require("react-docgen-typescript"));
const generateDocgenCodeBlock_1 = tslib_1.__importDefault(require("react-docgen-typescript-loader/dist/generateDocgenCodeBlock"));
/** Run the docgen parser and inject the result into the output */
function processModule(parser, webpackModule, tsProgram, loaderOptions) {
    if (!webpackModule) {
        return;
    }
    const componentDocs = parser.parseWithProgramProvider(webpackModule.userRequest, () => tsProgram);
    if (!componentDocs.length) {
        return;
    }
    const docs = generateDocgenCodeBlock_1.default(Object.assign({ filename: webpackModule.userRequest, source: webpackModule.userRequest, componentDocs }, loaderOptions)).substring(webpackModule.userRequest.length);
    // eslint-disable-next-line no-underscore-dangle
    let source = webpackModule._source._value;
    source += `\n${docs}\n`;
    // eslint-disable-next-line no-underscore-dangle, no-param-reassign
    webpackModule._source._value = source;
}
/** Get the contents of the tsconfig in the system */
function getTSConfigFile(tsconfigPath) {
    const basePath = path_1.default.dirname(tsconfigPath);
    const configFile = typescript_1.default.readConfigFile(tsconfigPath, typescript_1.default.sys.readFile);
    return typescript_1.default.parseJsonConfigFileContent(configFile.config, typescript_1.default.sys, basePath, {}, tsconfigPath);
}
/** Inject typescript docgen information into modules at the end of a build */
class DocgenPlugin {
    constructor(options) {
        this.name = "React Docgen Typescript Plugin";
        this.options = options;
    }
    apply(compiler) {
        const pathRegex = RegExp(`\\${path_1.default.sep}src.+\\.tsx`);
        const _a = this.options, { tsconfigPath, docgenCollectionName = "STORYBOOK_REACT_CLASSES", setDisplayName = true, typePropName = "type", compilerOptions: userCompilerOptions } = _a, docgenOptions = tslib_1.__rest(_a, ["tsconfigPath", "docgenCollectionName", "setDisplayName", "typePropName", "compilerOptions"]);
        let compilerOptions = {
            jsx: typescript_1.default.JsxEmit.React,
            module: typescript_1.default.ModuleKind.CommonJS,
            target: typescript_1.default.ScriptTarget.Latest,
        };
        if (userCompilerOptions) {
            compilerOptions = Object.assign(Object.assign({}, compilerOptions), userCompilerOptions);
        }
        if (tsconfigPath) {
            const { options } = getTSConfigFile(tsconfigPath);
            compilerOptions = Object.assign(Object.assign({}, compilerOptions), options);
        }
        compiler.hooks.make.tap(this.name, (compilation) => {
            compilation.hooks.seal.tap(this.name, () => {
                const modulesToProcess = [];
                compilation.modules.forEach((module) => {
                    // Skip ignored / external modules
                    if (!module.built || module.external || !module.rawRequest) {
                        return;
                    }
                    if (pathRegex.test(module.request)) {
                        modulesToProcess.push(module);
                    }
                });
                const parser = (tsconfigPath &&
                    docGen.withCustomConfig(tsconfigPath, docgenOptions)) ||
                    docGen.withCompilerOptions(compilerOptions, docgenOptions);
                const tsProgram = typescript_1.default.createProgram(modulesToProcess.map((v) => v.userRequest), compilerOptions);
                modulesToProcess.forEach((m) => processModule(parser, m, tsProgram, {
                    docgenCollectionName,
                    setDisplayName,
                    typePropName,
                }));
            });
        });
    }
}
exports.default = DocgenPlugin;
//# sourceMappingURL=index.js.map